#include "Frame.mqh"

#define FRAME_OFFSET 260
#define FRAME_FILTER_NAME_OFF ""
#define FRAME_FILTER_ID_OFF ULONG_MAX

class FRAMES
{
private:
  const int handle;

  string FilterName;
  ulong FilterID;

public:
  FRAMES( const string FileName = NULL ) : handle(::FileOpen(FileName, FILE_BIN | FILE_SHARE_WRITE | FILE_SHARE_READ | FILE_READ)),
                                           FilterName(FRAME_FILTER_NAME_OFF), FilterID(FRAME_FILTER_ID_OFF)
  {
    this.FrameFirst();
  }

  ~FRAMES( void )
  {
    if (this.handle != INVALID_HANDLE)
      ::FileClose(this.handle);
  }

  bool FrameFirst( void ) const
  {
    return((this.handle != INVALID_HANDLE) ? ::FileSeek(this.handle, FRAME_OFFSET, SEEK_SET) : ::FrameFirst());
  }

  bool FrameFilter( const string Name = FRAME_FILTER_NAME_OFF, const long ID = FRAME_FILTER_ID_OFF )
  {
    const bool Res = (this.handle != INVALID_HANDLE) || ::FrameFilter(Name, ID);

    if (this.handle != INVALID_HANDLE)
    {
      this.FilterName = Name;
      this.FilterID = ID;

      this.FrameFirst();
    }

    return(Res);
  }

  template <typename T>
  bool FrameNext( ulong &Pass, string &Name, long &ID, double &Value, T &Data[] ) const
  {
    bool Res = (this.handle != INVALID_HANDLE) || ::FrameNext(Pass, Name, ID, Value, Data);

    if (this.handle != INVALID_HANDLE)
    {
      FRAME Frame = {0};

      do
      {
        Res = ::FileSeek(this.handle, Frame.SizeOfArray, SEEK_CUR) && (::FileReadStruct(this.handle, Frame) == sizeof(Frame));
      }
      while (Res && (this.FilterName == FRAME_FILTER_NAME_OFF ? ((this.FilterID != FRAME_FILTER_ID_OFF) && (Frame.ID != this.FilterID))
                                                                  : (::ShortArrayToString(Frame.String) != this.FilterName)));

      if (Res && (Res = Frame.GetArray(handle, Data)))
      {
        Pass = Frame.Pass;
        Name = ::ShortArrayToString(Frame.String);
        ID = Frame.ID;
        Value = Frame.Value;
      }
    }

    return(Res);
  }

  bool FrameNext( ulong &Pass, string &Name, long &ID, double &Value ) const
  {
    uchar Data[];

    return((this.handle != INVALID_HANDLE) ? this.FrameNext(Pass, Name, ID, Value, Data) : ::FrameNext(Pass, Name, ID, Value));
  }

  static int GetFileNames( string &FileNames[], const string Folder = "Tester\\" )
  {
    string FileName;

    ::ArrayFree(FileNames);

    const long Handle = ::FileFindFirst(Folder + "*.mqd", FileName);

    do
    {
      FileNames[::ArrayResize(FileNames, ::ArraySize(FileNames) + 1) - 1] = Folder + FileName;
    }
    while (::FileFindNext(Handle, FileName));

    ::FileFindClose(Handle);

    return(::ArraySize(FileNames));
  }
};

#undef OFFSET
#undef FRAME_FILTER_ID_OFF
#undef FRAME_FILTER_NAME_OFF